% top level file, loads the grammar file ('program42-mod2.pl') and
% BB's model checker ('modelChecker2.pl'); presupposes the presence in the same
% directory of 'exampleModels.pl' with a model number 1 that uses the logical
% expressions of the grammar of English in 'program42-mod2.pl'

% interface to the tree viewer ('tv.jar') written by Martin Lazarov,
% some glue code by Frank Richter


:- use_module(library(system), [exec/3]).

% load the default test grammar and the parser
:- consult('program42-mod2.pl').
:- consult('modelChecker2.pl').


%
% parse/0 repeatedly reads sentences from stdin, and parses
% them, retrieving all parse trees, which are then send to a tree viewer.
% The tree viewer is a Java program that reads trees, encoded as LISP
% S-expressions, from its input stream and displays them. The logical
% representation(s) are displayed directly in the Prolog shell as a list (LFs)
%
parse :-
    exec('java -jar tv.jar', [pipe(StdOut), std, std], _Pid),
    repeat,
        readln(S),        % defined for SICStus below
        % initialize the tree counter
        retractall(tree_cnt_(_)), assert(tree_cnt_(0)),
        % retrieve all parse trees, incrementing the tree
        % counter on each
        findall(LF,
            (parse(S,Tree,LF), show_tree(StdOut,Tree), inc_tree_count),LFs),
        nl,
	write('The sentence receives the following truth conditions:'),
	nl,
	write(LFs),
	nl, nl,
	evaluate_lfs(LFs),
	nl,
        fail.

evaluate_lfs([]).
evaluate_lfs([LF|Ts]) :-
	write('Evaluating '),
	write(LF),
	write(' in Model 1:'),
	nl,
	evaluate(LF,1),     % evaluate FOL representation in model 1
	nl,
	evaluate_lfs(Ts).


%
% Increments the global tree counter. The counter is reset
% before each sentence is parsed.
%
inc_tree_count :-
    tree_cnt_(X),
    retractall(tree_cnt_(_)),
    X1 is X + 1,
    assert(tree_cnt_(X1)),
    write(X1), write(' '), !.

%
% Sends a tree to the tree viewer.
%
show_tree(Tree) :- show_tree(treg,Tree).
show_tree(Stream,Tree) :-
  % send the window's title
  write(Stream, Tree), nl(Stream),
  
  % send the tree
  print_tree(Stream,Tree), nl(Stream).


%
% print_tree/2 writes a tree represented as a Prolog term to
% the tree viewer's input stream in its format.
%

% leaf case
print_tree(Stream,Leaf) :-
    functor(Leaf,_,0), !,
    write(Stream, '("'),
    write(Stream, Leaf),
    write(Stream, '" [style=bolditalic color=blue])').

% internal node case
print_tree(Stream,Tree) :-
    functor(Tree,Root,Arr),
    write(Stream, '("'),
    write(Stream, Root),
    write(Stream, '" [style=bold]'),
    print_args(Stream,Tree,0,Arr),
    write(Stream, ')').

print_args(_,_,N,N) :- !.
print_args(Stream,Tree,I,N) :-
    I1 is I+1,
    arg(I1,Tree,Arg),
    print_tree(Stream,Arg), 
    print_args(Stream,Tree,I1,N).



% read_sent/1 reads in strings and tokenizes them into a list of
% atoms; assumes lower case letters and no punctuation in the input
% (taken from Pereira & Shieber 87)

readln(Line) :- read_sent(Line), !.

read_sent(Words) :-
	get0(Char),
	read_sent(Char,Words).


read_sent(C, []) :- newline(C), !.

read_sent(C, Words) :- space(C), !,
	get0(Char),
	read_sent(Char,Words).

read_sent(Char,[Word|Words]) :-
	read_word(Char,Chars,Next),
	name(Word,Chars),
	read_sent(Next,Words).


read_word(C, [], C) :- space(C), !.

read_word(C, [], C) :- newline(C), !.

read_word(Char, [Char|Chars], Last) :-
	get0(Next),
	read_word(Next,Chars,Last).

space(32).
newline(10).


